package net.nicoptere.scribbler;

import java.util.ArrayList;

import android.content.Context;
import android.graphics.Bitmap;
import android.graphics.BitmapFactory;
import android.graphics.Canvas;
import android.graphics.Paint;
import android.graphics.Rect;
import android.view.MotionEvent;
import android.view.View;
import android.view.View.OnTouchListener;

public class Scribbler extends View implements OnTouchListener
{
	private ArrayList<Point> points = new ArrayList<Point>();
	private ArrayList<Float> lines = new ArrayList<Float>();
	
	private int[] colors = {0xf48b49,0xf48b46,0xf48c43,0xf48c40,0xf38c3d,0xf38d3a,0xf38d37,0xf38d34,0xf38d30,0xf38e2d,0xf28e2a,0xf28e27,0xf28f24,0xf28f21,0xf28f1e,0xf2901b,0xf29018,0xf19015,0xf19112,0xf1910f,0xf29515,0xf39b1e,0xf5a227,0xf6a72f,0xf8ad38,0xf9b341,0xfab94a,0xfcbf53,0xfdc55b,0xffcb64,0xffcb64,0xffca62,0xffc95f,0xffc75c,0xffc65a,0xffc558,0xffc355,0xffc252,0xffc150,0xffc04e,0xffbe4b,0xffbd48,0xffbc46,0xffbb43,0xffb941,0xffb83e,0xffb73c,0xffb639,0xffb437,0xffb334,0xffb232,0xffb12f,0xffaf2d,0xffae2a,0xffad28,0xffac25,0xffaa23,0xffa920,0xffa81e,0xffa71b,0xffa518,0xffa416,0xffa314,0xffa211,0xffa00e,0xff9f0c,0xff9e0a,0xff9c07,0xff9b04,0xff9a02,0xff9900,0xfd9800,0xfb9700,0xf99600,0xf79500,0xf59400,0xf39300,0xf19200,0xef9100,0xee9000,0xec8f00,0xea8e00,0xe88d00,0xe68d00,0xe48c00,0xe28b00,0xe08a00,0xdf8900,0xdd8800,0xdb8700,0xd98600,0xd78500,0xd58400,0xd38300,0xd18200,0xcf8100,0xcd8000,0xcc7f00,0xca7e00,0xc87d00,0xc67c00,0xc47c00,0xc27b00,0xc07a00,0xbf7900,0xbd7800,0xbb7700,0xb97600,0xb77500,0xb57400,0xb37300,0xb17200,0xaf7100,0xad7000,0xac6f00,0xaa6e00,0xa86d00,0xa66c00,0xa46b00,0xa26b00,0xa06a00,0x9e6900,0x9d6800,0x9b6700,0x996600};
	private int colorId = 0;

	private Paint paint;
	private Bitmap background;
	
	public Scribbler( Context context )
	{
		super(context);
		
		//setting controls
		setFocusable(true);
		setFocusableInTouchMode(true);
		this.setOnTouchListener(this);
		
		//creates paint attributes
		paint = new Paint();
		paint.setAntiAlias(true);
		background = BitmapFactory.decodeResource( getResources(), R.drawable.background);
		
	}

	//renders the lines
	public void onDraw( Canvas canvas )
	{
		//draws the background image
		Rect rect = new Rect( 0,0, getWidth(), getHeight());
		canvas.drawBitmap(background, null, rect, null);
		
		int i = colorId = 0;
		for( i = 0; i < lines.size(); i += 4 )
		{
			//grabs the next color
			nextColor();
			//draws a line
			canvas.drawLine(lines.get(i), lines.get((i + 1)), lines.get((i + 2)), lines.get((i + 3)), paint);
		}

	}

	public boolean onTouch(View view, MotionEvent event)
	{
		
		if( event.getAction() == MotionEvent.ACTION_MOVE )
		{
			//adds a point to the points list
			Point pt = new Point(event.getX(),event.getY());
			points.add( pt );
			
			//and tests its distance against the existing points
			float dx, dy;
			for( Point p : points )
			{
				dx = pt.x - p.x;
				dy = pt.y - p.y;
				
				//if the distance is smaller than 50px ( 2500 = 50^2 )
				//gives 50% chance to create line ( Math.random() > .5 = 50 % )
				if( Math.abs((dx * dx + dy * dy)) < 2500 && Math.random() > .5 )
				{
					lines.add(pt.x);
					lines.add(pt.y);
					lines.add(p.x);
					lines.add(p.y);
				}
			}
		}
		
		//flush the view, call for onDraw and wait
		invalidate();

		return true;
	}
	
	//assigns the next color to the canvas
	private void nextColor()
	{
		//picking the next int
		int c = 0xFF << 24 | colors[ colorId ++ % colors.length ];
		
		//converting int to ARGB floats
		paint.setARGB( 	0xFF, ( c>>16 & 0xFF ), ( c>>8 & 0xFF ), ( c & 0xFF ) 	);
	}
	
	//deletes points and lines
	public void reset()
	{
		for( Point p : points )
		{
			p = null; 
		}
		points = new ArrayList<Point>();
		lines = new ArrayList<Float>();
		//clear the view
		invalidate();
	}
	
	//point dataholder
	private class Point
	{
		public float x, y;
		public Point( float x, float y )
		{
			this.x = x;
			this.y = y;
		}
	}
}
